package com.idea.buildManage.excel;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.idea.buildManage.model.MnpBuilding;
import com.idea.buildManage.service.MnpBuildingService;
import com.idea.buildManage.service.ParkFloorDiscService;
import com.idea.buildManage.service.ParkInfoService;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

public class MnpBuildingListener extends AnalysisEventListener<MnpBuildingExcel> {

    private List<MnpBuilding> list = new ArrayList<>();

    private List<MnpBuildingExcel> excelList =  new ArrayList<>();

    private List<MnpBuildingExcel> badList = new ArrayList<>();

    Integer count;

    private JSONObject jsonObject;

    private MnpBuildingService buildingService;
    private ParkFloorDiscService floorDiscService;
    private ParkInfoService parkInfoService;
    private SysDictService sysDictService;


    public MnpBuildingListener(MnpBuildingService buildingService,ParkFloorDiscService floorDiscService,
                               ParkInfoService parkInfoService,SysDictService sysDictService, JSONObject jsonObject) {
        this.buildingService = buildingService;
        this.floorDiscService = floorDiscService;
        this.parkInfoService = parkInfoService;
        this.sysDictService = sysDictService;
        this.jsonObject = jsonObject;
    }


    @Override
    public void invoke(MnpBuildingExcel mnpBuildingExcel, AnalysisContext analysisContext) {
        excelList.add(mnpBuildingExcel);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {


        // 入驻类型字典
        List<Map<String, String>> buildStructure = sysDictService.selectDictList("BUILD_STRUCTURE");
        if(CollectionUtils.isNotEmpty(excelList)){
            MnpBuilding data;
            MnpBuildingExcel excel = null;
            Map<String, String> discNameIdMap = floorDiscService.getNameIdMap();
            Map<String, String> parkNameIdMap = parkInfoService.getNameIdMap();
            for(int i = 0; i < excelList.size(); i++){

                boolean groupNameFlag = true;
                boolean discNameFlag = true;
                boolean buildNumFlag = true;
                boolean constructionNumFlag = true;
                boolean areaFlag = true;
                boolean residentialAreaFlag = true;
                boolean unResidentialAreaFlag = true;
                boolean coverAnAreaFlag = true;
                boolean undergroundAreaFlag = true;
                boolean roomNumFlag = true;
                boolean numberInfoFlag = true;
                boolean floorsFlag = true;
                boolean overgroundFloorsFlag = true;
                boolean undergroundFloorsFlag = true;
                boolean structureFlag = true;
                boolean regionalismCodeFlag = true;
                boolean landCodeFlag = true;
                StringBuilder builder = new StringBuilder();

                try {
                    data = new MnpBuilding();
                    excel = excelList.get(i);
                    if (StringUtils.isEmpty(excel.getGroupName())) {
                        groupNameFlag = false;
                        builder.append("楼盘/小区名称为空").append(",");
                        excel.setGroupName("{" + excel.getGroupName() + "}");
                    }
                    data.setGroupName(excel.getGroupName());

                    if (StringUtils.isEmpty(excel.getDiscName())) {
                        discNameFlag = false;
                        builder.append("分期名称为空").append(",");
                        excel.setDiscName("{" + excel.getDiscName() + "}");
                    }
                    data.setDiscName(excel.getDiscName());

                    // 判断小区是否存在
                    if(StringUtils.isEmpty(parkNameIdMap.get(excel.getGroupName()))){
                        groupNameFlag = false;
                        builder.append("楼盘/小区不存在").append(",");
                        excel.setGroupName("{" + excel.getGroupName() + "}");
                    }
                    data.setGroupId(parkNameIdMap.get(excel.getGroupName()));

                    // 判断分期是否存在
                    if(StringUtils.isEmpty(discNameIdMap.get(excel.getDiscName()))){
                        discNameFlag = false;
                        builder.append("分期不存在").append(",");
                        excel.setDiscName("{" + excel.getDiscName() + "}");
                    }
                    data.setDiscId(discNameIdMap.get(excel.getDiscName()));

                    // 楼栋号
                    if (StringUtils.isEmpty(excel.getBuildNum())) {
                        buildNumFlag = false;
                        builder.append("单元楼栋号为空").append(",");
                        excel.setBuildNum("{" + excel.getBuildNum() + "}");
                    }
                    data.setBuildNum(excel.getBuildNum());

                    // 施工号
                    if (StringUtils.isEmpty(excel.getConstructionNum())) {
                        constructionNumFlag = false;
                        builder.append("施工号为空").append(",");
                        excel.setConstructionNum("{" + excel.getConstructionNum() + "}");
                    }
                    data.setConstructionNum(excel.getConstructionNum());

                    // 总面积
                    if (StringUtils.isEmpty(excel.getArea())) {
                        areaFlag = false;
                        builder.append("总面积为空").append(",");
                        excel.setArea("{" + excel.getArea() + "}");
                    }
                    try {
                        data.setArea(Double.valueOf(excel.getArea()));
                    }catch (NumberFormatException e){
                        areaFlag = false;
                        builder.append("总面积格式有误").append(",");
                        excel.setArea("{" + excel.getArea() + "}");
                    }

                    // 住宅面积
                    if (StringUtils.isEmpty(excel.getResidentialArea())) {
                        residentialAreaFlag = false;
                        builder.append("住宅面积为空").append(",");
                        excel.setResidentialArea("{" + excel.getResidentialArea() + "}");
                    }
                    try {
                        data.setResidentialArea(Double.valueOf(excel.getResidentialArea()));
                    }catch (NumberFormatException e){
                        residentialAreaFlag = false;
                        builder.append("住宅面积格式有误").append(",");
                        excel.setResidentialArea("{" + excel.getResidentialArea() + "}");
                    }

                    // 非住宅面积
                    if (StringUtils.isEmpty(excel.getUnResidentialArea())) {
                        unResidentialAreaFlag = false;
                        builder.append("非住宅面积为空").append(",");
                        excel.setUnResidentialArea("{" + excel.getUnResidentialArea() + "}");
                    }
                    try {
                        data.setUnResidentialArea(Double.valueOf(excel.getUnResidentialArea()));
                    }catch (NumberFormatException e){
                        unResidentialAreaFlag = false;
                        builder.append("非住宅面积格式有误").append(",");
                        excel.setUnResidentialArea("{" + excel.getUnResidentialArea() + "}");
                    }

                    // 占地面积
                    if (StringUtils.isEmpty(excel.getCoverAnArea())) {
                        coverAnAreaFlag = false;
                        builder.append("占地面积为空").append(",");
                        excel.setCoverAnArea("{" + excel.getCoverAnArea() + "}");
                    }
                    try {
                        data.setCoverAnArea(Double.valueOf(excel.getCoverAnArea()));
                    }catch (NumberFormatException e){
                        coverAnAreaFlag = false;
                        builder.append("占地面积格式有误").append(",");
                        excel.setCoverAnArea("{" + excel.getCoverAnArea() + "}");
                    }

                    // 地下面积
                    if (StringUtils.isEmpty(excel.getUndergroundArea())) {
                        undergroundAreaFlag = false;
                        builder.append("地下面积为空").append(",");
                        excel.setUndergroundArea("{" + excel.getUndergroundArea() + "}");
                    }
                    try {
                        data.setUndergroundArea(Double.valueOf(excel.getUndergroundArea()));
                    }catch (NumberFormatException e){
                        undergroundAreaFlag = false;
                        builder.append("地下面积格式有误").append(",");
                        excel.setUndergroundArea("{" + excel.getUndergroundArea() + "}");
                    }

                    // 房号
                    if (StringUtils.isEmpty(excel.getRoomNum())) {
                        roomNumFlag = false;
                        builder.append("房号为空").append(",");
                        excel.setRoomNum("{" + excel.getRoomNum() + "}");
                    }
                    data.setRoomNum(excel.getRoomNum());

                    // 套数
                    if (StringUtils.isEmpty(excel.getNumberInfo())) {
                        numberInfoFlag = false;
                        builder.append("套数为空").append(",");
                        excel.setNumberInfo("{" + excel.getNumberInfo() + "}");
                    }
                    data.setNumberInfo(excel.getNumberInfo());

                    // 总层数
                    if (StringUtils.isEmpty(excel.getFloors())) {
                        floorsFlag = false;
                        builder.append("总层数为空").append(",");
                        excel.setFloors("{" + excel.getFloors() + "}");
                    }
                    try {
                        data.setFloors(Integer.valueOf(excel.getFloors()));
                    }catch (NumberFormatException e){
                        floorsFlag = false;
                        builder.append("总层数格式有误").append(",");
                        excel.setFloors("{" + excel.getFloors() + "}");
                    }

                    // 地上层数
                    if (StringUtils.isEmpty(excel.getOvergroundFloors())) {
                        overgroundFloorsFlag = false;
                        builder.append("地上层数为空").append(",");
                        excel.setOvergroundFloors("{" + excel.getOvergroundFloors() + "}");
                    }
                    try {
                        data.setOvergroundFloors(Integer.valueOf(excel.getOvergroundFloors()));
                    }catch (NumberFormatException e){
                        overgroundFloorsFlag = false;
                        builder.append("地上层数格式有误").append(",");
                        excel.setOvergroundFloors("{" + excel.getOvergroundFloors() + "}");
                    }

                    // 地下层数
                    if (StringUtils.isEmpty(excel.getUndergroundFloors())) {
                        undergroundFloorsFlag = false;
                        builder.append("地下层数为空").append(",");
                        excel.setUndergroundFloors("{" + excel.getUndergroundFloors() + "}");
                    }
                    try {
                        data.setUndergroundFloors(Integer.valueOf(excel.getUndergroundFloors()));
                    }catch (NumberFormatException e){
                        undergroundFloorsFlag = false;
                        builder.append("地下层数格式有误").append(",");
                        excel.setUndergroundFloors("{" + excel.getUndergroundFloors() + "}");
                    }

                    // 结构
                    if (StringUtils.isEmpty(excel.getStructure())) {
                        structureFlag = false;
                        builder.append("结构为空").append(",");
                        excel.setStructure("{" + excel.getStructure() + "}");
                    }
                    for (Map<String, String> stringStringMap : buildStructure) {
                        if (stringStringMap.get("label").equals(excel.getStructure())) {
                            data.setStructure(stringStringMap.get("value"));
                        }
                    }
                    // 没匹配上的情况
                    if(StringUtils.isEmpty(data.getStructure())){
                        structureFlag = false;
                        builder.append("结构格式有误").append(",");
                        excel.setStructure("{" + excel.getStructure() + "}");
                    }

//                    // 行政区划
//                    if (StringUtils.isEmpty(excel.getRegionalismCode())) {
//                        regionalismCodeFlag = false;
//                        builder.append("行政区划为空").append(",");
//                        excel.setRegionalismCode("{" + excel.getRegionalismCode() + "}");
//                    }
//                    data.setRegionalismCode(excel.getRegionalismCode());

                    // 地号
                    if (StringUtils.isEmpty(excel.getLandCode())) {
                        landCodeFlag = false;
                        builder.append("地号为空").append(",");
                        excel.setLandCode("{" + excel.getLandCode() + "}");
                    }
                    data.setLandCode(excel.getLandCode());


                    if (groupNameFlag && discNameFlag && buildNumFlag && constructionNumFlag && areaFlag && residentialAreaFlag
                            && unResidentialAreaFlag && coverAnAreaFlag && undergroundAreaFlag && roomNumFlag && numberInfoFlag
                            && floorsFlag && overgroundFloorsFlag && undergroundFloorsFlag && structureFlag && regionalismCodeFlag
                            && landCodeFlag
                    ) {
                        data.setRemark(excel.getRemark());
                        data.setPriceRemark(excel.getPriceRemark());
                        data.setId(IdUtil.simpleUUID());
                        data.setCreatedId(ShiroUtils.getUserId());
                        data.setCreatedAt(new Date());
                        list.add(data);
                    }else {
                        throw new RuntimeException(builder.toString());
                    }

                }catch (Exception e){
                    e.printStackTrace();
                    String message = e.getMessage();
                    excel.setReason(message);
                    badList.add(excel);
                }
            }

            count = list.size();
            jsonObject.put("successCount", count);
            jsonObject.put("errorList", badList);
            jsonObject.put("errorCount", badList.size());
            //保存
            saveData();

            excelList.clear();
            list.clear();
        }else {
            throw new RuntimeException("excel无数据");
        }

    }

    private void saveData(){
        if (CollectionUtils.isNotEmpty(list)) {
            buildingService.insertBatch(list);
        }
    }


}
