package com.idea.buildManage.service;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.dto.MnpBuildingDto;
import com.idea.buildManage.excel.MnpBuildingExcel;
import com.idea.buildManage.excel.ParkInfoExcel;
import com.idea.buildManage.excel.WriteHandlerStrategy;
import com.idea.buildManage.mapper.MnpBuildingExtendMapper;
import com.idea.buildManage.mapper.MnpBuildingMapper;
import com.idea.buildManage.model.*;
import com.idea.invoice.util.InvoiceUtil;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.mapper.FrameDataMapper;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MnpBuildingService implements BaseService<MnpBuilding, MnpBuildingExample> {

    //文件mapper
    @Autowired
    private MnpBuildingMapper modelMapper;
    @Autowired
    private FrameDataMapper frameDataMapper;
    @Autowired
    private ParkRoomService parkRoomService;
    @Autowired
    private MnpBuildingExtendMapper extendMapper;
    
    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<MnpBuildingDto> list(Tablepar tablepar, MnpBuildingDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<MnpBuildingDto> list = extendMapper.listByModel(model);
        PageInfo<MnpBuildingDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<MnpBuilding> listAll(MnpBuilding model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private MnpBuildingExample getCondition(MnpBuilding model, DateTrans dt) {
        MnpBuildingExample ex = new MnpBuildingExample();
        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String id) {
        // 先查询该楼栋下是否存在房间
        ParkRoomExample parkRoomExample = new ParkRoomExample();
        parkRoomExample.createCriteria().andBuildIdEqualTo(id);
        long count = parkRoomService.countByExample(parkRoomExample);
        if (count > 0) {
            throw new RuntimeException("该楼栋下存在房间,无法删除!");
        }
        return modelMapper.deleteByPrimaryKey(id);
    }

    @Override
    public MnpBuilding selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(MnpBuilding record) {
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(MnpBuilding record, MnpBuildingExample example) {
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(MnpBuilding record, MnpBuildingExample example) {
        return modelMapper.updateByExample(record, example);
    }

    public int updateByPrimaryKey(MnpBuilding record) {
        return modelMapper.updateByPrimaryKey(record);
    }

    @Override
    public List<MnpBuilding> selectByExample(MnpBuildingExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(MnpBuildingExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(MnpBuilding model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model, dt));
    }

    @Override
    public int deleteByExample(MnpBuildingExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(MnpBuilding record) {
        record.setId(IdUtil.simpleUUID());
        return insertWithoutId(record);
    }

    public int insertWithoutId(MnpBuilding record) {
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(MnpBuilding record) {
        record.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(record);
    }


    public List<MnpBuildingDto> excelListAll(MnpBuildingDto model) {

        List<MnpBuildingDto> list = extendMapper.listByModel(model);
        return list;
    }

    /**
     * 根据合同id查询楼栋
     * @param contractId
     * @return
     */
    public List<MnpBuilding> selectByContractId(String contractId){
       return extendMapper.selectByContractId(contractId);
    }

    /**
     * 批量插入
     * @param list
     */
    public void insertBatch(List<MnpBuilding> list){
        extendMapper.batchInsert(list);
    }

    public Map<String,String> getNameIdMap() {
        Map<String, String> map = new HashMap<>();
        List<MnpBuilding> list = selectByExample(new MnpBuildingExample());
        if (CollectionUtils.isNotEmpty(list)) {
            map = list.stream().collect(Collectors.toMap(MnpBuilding::getBuildNum, MnpBuilding::getId, (v1, v2) ->v1));
        }
        return map;
    }

    /**
     * 楼栋删除
     * @param id
     * @return
     */
    public int delete(String id){
        ParkRoomExample roomExample = new ParkRoomExample();
        roomExample.createCriteria().andBuildIdEqualTo(id);
        List<ParkRoom> parkRooms = parkRoomService.selectByExample(roomExample);
        if(CollectionUtils.isNotEmpty(parkRooms)){
            throw new RuntimeException("该数据有子房间存在,无法删除");
        }
        return deleteByPrimaryKey(id);
    }

    public String getOrderNum(MnpBuilding model){
        MnpBuildingExample example = new MnpBuildingExample();
        example.createCriteria().andDiscIdEqualTo(model.getDiscId());
        Long count = countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "001";
        }else {
            count++;
            number = String.format("%03d", count);
        }
        return number;
    }


    public void errorListExport(List<MnpBuildingExcel> list, HttpServletResponse response) throws IOException {


        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("失败记录" + date, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");

        String templateFileName = "/home/project/settleDown/tem/楼盘楼栋导出模版.xlsx";
        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream())
                .registerWriteHandler(new WriteHandlerStrategy())
                .withTemplate(templateFileName)
                .build();
        WriteSheet writeSheet = EasyExcel.writerSheet()
                .build();
        FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();

        // 填充数据
        excelWriter.fill(list, fillConfig, writeSheet);
        excelWriter.finish();

    }

    /**
     * 新增载体
     * @param record
     * @return
     */
    public int add(MnpBuilding record){

        // 同一楼盘下 载体名称不能重复
        MnpBuildingExample mnpBuildingExample = new MnpBuildingExample();
        mnpBuildingExample.createCriteria().andBuildNumEqualTo(record.getBuildNum()).andDiscIdEqualTo(record.getDiscId());
        long count = countByExample(mnpBuildingExample);
        if (count > 0) {
            throw new RuntimeException(record.getBuildNum() + "已存在,请调整后重新添加!!!");
        }
        String id = IdUtil.simpleUUID();
        record.setCreatedId(ShiroUtils.getUser().getId());
        record.setCreatedAt(new Date());
        String orderNum = getOrderNum(record);
        record.setOrderNum(orderNum);
        record.setId(id);
        int result = insertWithoutId(record);

//        try {
//            JSONObject json = new JSONObject();
//            json.put("srcsystemid", id);
//            json.put("code", id);
//            json.put("name", record.getBuildNum());
//            json.put("pk_org", InvoiceUtil.pk_org);
//            json.put("pk_defdoclist", InvoiceUtil.pk_defdoclist_build);
//            log.info("推送载体详情:{}", json.toJSONString());
//            // 给nc推送载体数据
//            String resultJson = InvoiceUtil.defdocAdd(json);
//            JSONObject jsonObject = JSONObject.parseObject(resultJson);
//            String success = jsonObject.getString("success");
//            if (success.equals("true")) {
//                JSONObject data = jsonObject.getJSONObject("data");
//                String code = data.getString("code");
//                String ncid = data.getString("ncid");
//                MnpBuilding model = new MnpBuilding();
//                model.setId(id);
//                model.setNcid(ncid);
//                model.setNcCode(code);
//                updateByPrimaryKeySelective(model);
//            }
//        } catch (Exception e) {
//            log.error("推送载体失败", e);
//        }

        return result;
    }

    public int edit(MnpBuilding model){

        int result = updateByPrimaryKeySelective(model);

//        try {
//            MnpBuilding building = selectByPrimaryKey(model.getId());
//            JSONObject json = new JSONObject();
//            json.put("ncid", building.getNcid());
//            json.put("code", building.getNcCode());
//            json.put("name", building.getBuildNum());
//            // 给nc推送修改载体数据
//            InvoiceUtil.defdocUpdate(json);
//        }catch (Exception e){
//            log.error("推送修改载体失败",e);
//        }

        return result;
    }

    public List<MnpBuilding> selectNeedSendList(){
        return extendMapper.selectNeedSendList();
    }



    public void downLoadTemplate(HttpServletResponse response) throws IOException {

        File file = new File("/home/project/settleDown/tem/楼盘楼栋导入模版.xlsx");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("楼盘楼栋导入模版" + date, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
        OutputStream out = response.getOutputStream();
        IoUtil.copy(FileUtil.getInputStream(file), out);
        out.close();
    }









}
