package com.idea.buildManage.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.poi.excel.ExcelUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.excel.MnpBuildingExcel;
import com.idea.buildManage.excel.ParkRoomExcel;
import com.idea.buildManage.excel.WriteHandlerStrategy;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.*;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.RoomSelectionInfo;
import com.idea.customerManagement.model.RoomSelectionInfoExample;
import com.idea.invoice.util.InvoiceUtil;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.conf.V2Config;
import com.rockstar.common.support.Convert;
import com.rockstar.frame.mapper.FrameDataMapper;
import com.rockstar.frame.model.FrameData;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.SnowflakeIdWorker;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.beans.Transient;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class ParkRoomService implements BaseService<ParkRoom, ParkRoomExample> {

    //文件mapper
    @Autowired
    private SysDictService dictService;
    @Autowired
    private ParkRoomMapper modelMapper;
    @Autowired
    private FrameDataMapper frameDataMapper;
    @Autowired
    private ParkRoomExtendMapper extendMapper;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;
    @Autowired
    private MnpBuildingService buildingService;
    @Autowired
    private ParkFloorDiscService floorDiscService;
    @Autowired
    private ParkInfoService parkInfoService;

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkRoom> list(Tablepar tablepar, ParkRoom model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkRoom> list = modelMapper.selectByExample(getCondition(model, dt));
        PageInfo<ParkRoom> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 查询指定
     *
     * @return
     */
    public List<ParkRoom> listAll(ParkRoom model, DateTrans dt) {
        return modelMapper.selectByExample(getCondition(model, dt));
    }

    private ParkRoomExample getCondition(ParkRoom model, DateTrans dt) {
        ParkRoomExample ex = new ParkRoomExample();
        ex.setOrderByClause("order_num asc");
        ParkRoomExample.Criteria criteria = ex.createCriteria();
        if (StringUtils.isNotEmpty(model.getBuildId())) {
            criteria.andBuildIdEqualTo(model.getBuildId());
        }
        return ex;
    }

    @Override
    public int deleteByPrimaryKey(String ids) {
        List<String> lista = Convert.toListStrArray(ids);
        ParkRoomExample example = new ParkRoomExample();
        example.createCriteria().andIdIn(lista);
        return modelMapper.deleteByExample(example);
    }

    @Override
    public ParkRoom selectByPrimaryKey(String id) {
        return modelMapper.selectByPrimaryKey(id);
    }

    @Override
    public int updateByPrimaryKeySelective(ParkRoom record) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int updateByExampleSelective(ParkRoom record, ParkRoomExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExampleSelective(record, example);
    }

    @Override
    public int updateByExample(ParkRoom record, ParkRoomExample example) {
        record.setUpdatedAt(new Date());
        return modelMapper.updateByExample(record, example);
    }

    @Override
    public List<ParkRoom> selectByExample(ParkRoomExample example) {
        return modelMapper.selectByExample(example);
    }

    @Override
    public long countByExample(ParkRoomExample example) {
        return modelMapper.countByExample(example);
    }

    public long countByExample(ParkRoom model, DateTrans dt) {
        return modelMapper.countByExample(getCondition(model, dt));
    }

    @Override
    public int deleteByExample(ParkRoomExample example) {
        return modelMapper.deleteByExample(example);
    }

    public int insert(ParkRoom record) {

        record.setId(SnowflakeIdWorker.getUUID());
        return insertWithoutId(record);
    }

    public int edit(ParkRoom model) {

        model.setUpdatedAt(new Date());
        model.setUpdatedId(ShiroUtils.getUserId());
//        try {
//            ParkRoom parkRoom = selectByPrimaryKey(model.getId());
//            JSONObject json = new JSONObject();
//            json.put("ncid", parkRoom.getNcid());
//            json.put("code", parkRoom.getNcCode());
//            json.put("name", parkRoom.getRoomNo());
//            log.info("推送房间修改详情:{}", json.toJSONString());
//            // 给nc推送修改载体数据
//            InvoiceUtil.defdocUpdate(json);
//        }catch (Exception e){
//            log.error("推送修改房间失败",e);
//        }
        return updateByPrimaryKeySelective(model);
    }

    public int insertWithoutId(ParkRoom record) {
        record.setCreatedAt(new Date());
        return modelMapper.insert(record);
    }

    @Override
    public int insertSelective(ParkRoom record) {
        record.setId(SnowflakeIdWorker.getUUID());
        record.setCreatedAt(new Date());
        return modelMapper.insertSelective(record);
    }

    /**
     * 分页查询
     *
     * @return
     */
    public PageInfo<ParkRoomResponse> listByModel(Tablepar tablepar, ParkRoomResponse model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ParkRoomResponse> list = extendMapper.selectByModel(model);
        PageInfo<ParkRoomResponse> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 分页查询
     *
     * @return
     */
    public List<ParkRoomResponse> excelList(ParkRoomResponse model, DateTrans dt) {
        List<ParkRoomResponse> list = extendMapper.selectByModel(model);
        return list;
    }


    // 获取房屋面积
    public Double getAreaByIds(String ids) {

        List<String> idList = Arrays.asList(ids.split(","));
        ParkRoomExample example = new ParkRoomExample();
        example.createCriteria().andIdIn(idList);
        List<ParkRoom> parkRooms = modelMapper.selectByExample(example);
        BigDecimal area = new BigDecimal("0");

        return area.doubleValue();
    }

    /**
     * 批量插入
     * @param list
     */
    public void insertBatch(List<ParkRoom> list){
        extendMapper.insertBatch(list);
    }

    public int delete(String id){

        // 已收取 已转 的定金
        List<Integer> status = Arrays.asList(2,3);
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andHouseIdEqualTo(id).andStatusIn(status);
        List<RoomSelectionInfo> list = roomSelectionInfoMapper.selectByExample(example);
        if(CollectionUtils.isNotEmpty(list)){
            throw new RuntimeException("该房间已收取认购金，无法删除");
        }
        return deleteByPrimaryKey(id);
    }

    /**
     * 查询房屋全称
     * @param houseId
     * @return
     */
    public ParkRoomResponse getFullName(String houseId){
        ParkRoomResponse response = extendMapper.getFullName(houseId);
        return response;
    }

    public String getOrderNum(ParkRoom model) {
        ParkRoomExample example = new ParkRoomExample();
        example.createCriteria().andBuildIdEqualTo(model.getBuildId());
        Long count = countByExample(example);
        String number = "";
        if(count.doubleValue() == 0){
            number = "001";
        }else {
            count++;
            number = String.format("%03d", count);
        }
        return number;
    }

    public void errorListExport(List<ParkRoomExcel> list, HttpServletResponse response) throws IOException {


        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = sdf.format(new Date());
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        String fileName = URLEncoder.encode("失败记录" + date, "UTF-8");
        response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");

        String templateFileName = "/home/project/settleDown/tem/房间导出模板.xlsx";
        //工作薄对象
        ExcelWriter workBook = EasyExcel.write(response.getOutputStream(), ParkRoomExcel.class)
                .withTemplate(templateFileName)
                .registerWriteHandler(new WriteHandlerStrategy())
                .build();
        //工作区对象
        WriteSheet sheet = EasyExcel.writerSheet().build();
        FillConfig build = FillConfig.builder().forceNewRow(true).build();
        workBook.fill(list, build, sheet);
        workBook.finish();


    }

    /**
     * 新增房间
     * @param record
     * @return
     */
    public int add(ParkRoom record){

        String id = SnowflakeIdWorker.getUUID();
        record.setId(id);
        record.setCreatedAt(new Date());
        record.setCreatedId(ShiroUtils.getUserId());
        // 默认待售
        record.setSoldStatus(1);
        // 默认正常
        record.setSaleStatus(1);
        // 选房状态
        record.setIsChoose(0);
        String orderNum = getOrderNum(record);
        record.setOrderNum(orderNum);
        int result = insertWithoutId(record);

//        try {
//            ParkInfo parkInfo = parkInfoService.selectByPrimaryKey(record.getGroupId());
//            ParkFloorDisc parkFloorDisc = floorDiscService.selectByPrimaryKey(record.getDiscId());
//            MnpBuilding mnpBuilding = buildingService.selectByPrimaryKey(record.getBuildId());
//            String name = parkInfo.getGroupName() + "-" + parkFloorDisc.getName() + "-" + mnpBuilding.getBuildNum()
//                    + "-" + record.getRoomNo();
//            JSONObject json = new JSONObject();
//            json.put("srcsystemid", id);
//            json.put("code", id);
//            json.put("name", name);
//            json.put("pk_org", InvoiceUtil.pk_org);
//            json.put("pk_defdoclist", InvoiceUtil.pk_defdoclist_room);
//            // 给nc推送房间数据
//            String resultJson = InvoiceUtil.defdocAdd(json);
//            JSONObject jsonObject = JSONObject.parseObject(resultJson);
//            String success = jsonObject.getString("success");
//            if (success.equals("true")) {
//                JSONObject data = jsonObject.getJSONObject("data");
//                String code = data.getString("code");
//                String ncid = data.getString("ncid");
//                ParkRoom model = new ParkRoom();
//                model.setId(id);
//                model.setNcid(ncid);
//                model.setNcCode(code);
//                updateByPrimaryKeySelective(model);
//            }
//        } catch (Exception e) {
//            log.error("推送房间失败", e);
//        }
        return result;

    }

    public List<ParkRoom> selectNeedSendList(){
        return extendMapper.selectNeedSendList();
    }

    public ParkRoomResponse getById(String id){

        return extendMapper.getById(id);
    }

}
