package com.idea.paymentManagement.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkFloorDiscMapper;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkInfoExample;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.dto.IntentionalDepositDto;
import com.idea.customerManagement.mapper.ContractManageMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.customerManagement.service.BuyerService;
import com.idea.invoice.dto.InvoiceManageDto;
import com.idea.invoice.model.InvoiceManage;
import com.idea.invoice.model.InvoiceManageExample;
import com.idea.invoice.model.ReceiptManage;
import com.idea.invoice.model.ReceiptManageExample;
import com.idea.invoice.service.InvoiceManageService;
import com.idea.invoice.service.ReceiptManageService;
import com.idea.invoice.util.InvoiceConstant;
import com.idea.invoice.util.InvoiceUtil;
import com.idea.paymentManagement.dto.PayLogDto;
import com.idea.paymentManagement.mapper.PayLogExtendMapper;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.transactionRecordManage.mapper.ConvertRecordMapper;
import com.idea.transactionRecordManage.model.ConvertRecord;
import com.idea.util.DateUtils;
import com.idea.util.MoneyUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class PayLogService implements BaseService<PayLog, PayLogExample> {

    @Autowired
    private PayLogMapper modelMapper;
    @Autowired
    private PayLogExtendMapper extendMapper;
    @Autowired
    private ContractManageMapper contractManageMapper;
    @Autowired
    private FrameUserService frameUserService;
    @Autowired
    private InvoiceManageService invoiceManageService;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ReceiptManageService receiptManageService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private SysDictService sysDictService;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private ParkInfoMapper parkInfoMapper;
    @Autowired
    private ParkFloorDiscMapper parkFloorDiscMapper;
    @Autowired
    private ConvertRecordMapper convertRecordMapper;
    @Autowired
    private RoomSelectionInfoMapper roomSelectionInfoMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(PayLog PayLog) {
        PayLog.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(PayLog);
    }

    public int insertWithoutId(PayLog PayLog) {
        return modelMapper.insertSelective(PayLog);
    }

    @Override
    public PayLog selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(PayLog PayLog) {
        return modelMapper.updateByPrimaryKeySelective(PayLog);
    }

    @Override
    public int updateByExampleSelective(PayLog PayLog, PayLogExample PayLogExample) {
        return modelMapper.updateByExampleSelective(PayLog, PayLogExample);
    }

    @Override
    public int updateByExample(PayLog PayLog, PayLogExample PayLogExample) {
        return modelMapper.updateByExample(PayLog, PayLogExample);
    }

    @Override
    public List<PayLog> selectByExample(PayLogExample PayLogExample) {
        return modelMapper.selectByExample(PayLogExample);
    }

    @Override
    public long countByExample(PayLogExample PayLogExample) {
        return modelMapper.countByExample(PayLogExample);
    }

    @Override
    public int deleteByExample(PayLogExample PayLogExample) {
        return modelMapper.deleteByExample(PayLogExample);
    }


    public PageInfo<ContractManageDto> listByModel(Tablepar tablepar, ContractManageDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<ContractManageDto> list = extendMapper.listByModel(model);
        PageInfo<ContractManageDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }


    /**
     * 新增
     *
     * @param model
     * @return
     */
    @Transactional
    public int add(PayLog model,String userId) {

        if(StringUtils.isEmpty(userId)){
            userId = ShiroUtils.getUserId();
        }
        String serialNumber = getSerialNumber(model);
        String id = IdUtil.simpleUUID();
        model.setCreatedBy(userId);
        model.setCreatedAt(new Date());
        model.setStatus(0);
        model.setPaymentStatus(1);
        model.setId(id);
        model.setSerialNumber(serialNumber);
        int result = insertWithoutId(model);
        // 更新合同 预收款 收款状态
        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(model.getContractId());
        if (model.getPayType() == 1) {
            String receivedMoneyStr = extendMapper.getMoneyByContractId(model.getContractId(), model.getPayType(),null);
            BigDecimal buyerMoney = contractManage.getBuyerMoney();
            BigDecimal receivedMoney = new BigDecimal(receivedMoneyStr);
            if (receivedMoney.doubleValue() < buyerMoney.doubleValue()) {
                // 部分收款
                contractManage.setCollectionStatus(1);
            } else {
                // 全部收款
                contractManage.setCollectionStatus(2);

                // 查询该合同是否已生成 正式发票
                InvoiceManageExample invoiceManageExample = new InvoiceManageExample();
                invoiceManageExample.createCriteria().andTypeEqualTo("1")
                        .andContractIdEqualTo(model.getContractId())
                        .andPaymentTypeEqualTo("1");
                List<InvoiceManage> invoiceManages = invoiceManageService.selectByExample(invoiceManageExample);
                if (CollectionUtils.isEmpty(invoiceManages)) {
                    // 新增待推送的开票信息
                    InvoiceManageDto invoiceManage = invoiceManageService.getInfoByContract(model.getContractId(),
                            "*不动产*", 1);
                    // 正式发票
                    invoiceManage.setType("2");
                    // 房款
                    invoiceManage.setPaymentType("1");
                    invoiceManage.setSendStatus("0");
                    invoiceManage.setInvoiceStatus("0");
                    invoiceManage.setInvoiceTax(null);
                    invoiceManage.setRemark(null);
                    invoiceManage.setCreateUser(userId);
                    invoiceManage.setCreateTime(new Date());
                    invoiceManageService.insertSelective(invoiceManage);
                }
            }
        }
        contractManageMapper.updateByPrimaryKeySelective(contractManage);
        // 票据管理 新增一条记录
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(contractManage.getCustomerManagementId());
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        receiptManage.setBusinessId(id);
        // 票据类型
        String receiptType = "";
        switch (model.getContentType()) {
            case 1: // 定金
                receiptType = "2";
                break;
            case 2: // 首付
                receiptType = "3";
                break;
            case 3: // 银行按揭
                receiptType = "4";
                break;
            case 4: // 专项维修资金
                receiptType = "";
                break;
            case 5: // 楼款
                receiptType = "";
                break;
            case 6: // 补差款
                receiptType = "";
                break;
            case 7: // 意向金
                receiptType = "1";
                break;
        }
        receiptManage.setReceiptType(receiptType);
        receiptManage.setReceiptMoney(model.getPayMoney());
        receiptManage.setReceiptNumber(serialNumber);
        receiptManageService.add(receiptManage,userId);

        return result;
    }

    public String getSerialNumber(PayLog model) {
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        String groupId = customerManagement.getGroupId();
        String discId = customerManagement.getDiscId();
        Integer contentType = model.getContentType();
        String type = "";
        if (contentType == 2) {
            type = "SQ";
        }
        if (contentType == 3) {
            type = "YHAJ";
        }
        // 查找这个小区已收取 首期或银行按揭的 数据量
        Integer count = extendMapper.countByGroupId(groupId, String.valueOf(contentType));
        String number = "";
        if (count == 0) {
            number = "00001";
        } else {
            count++;
            number = String.format("%05d", count);
        }
        ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(groupId);
        ParkFloorDisc parkFloorDisc = parkFloorDiscMapper.selectByPrimaryKey(discId);
        String serialNumber = parkInfo.getGroupName() + parkFloorDisc.getName() + "-" + type + number;
        return serialNumber;
    }


    /**
     * 收款信息
     *
     * @param model
     * @param dt
     * @return
     */
    public List<PayLog> payListAll(PayLog model, DateTrans dt) {
        PayLogExample condition = getCondition(model, dt);
        // 剔除意向金
        PayLogExample.Criteria criteria = condition.getOredCriteria().get(0);
        criteria.andContentTypeNotEqualTo(7)
                .andStatusEqualTo(0);
        return modelMapper.selectByExample(condition);
    }

    private PayLogExample getCondition(PayLog model, DateTrans dt) {
        PayLogExample ex = new PayLogExample();
        ex.setOrderByClause("created_at asc");
        PayLogExample.Criteria criteria = ex.createCriteria();
        if (StringUtils.isNotEmpty(model.getContractId())) {
            criteria.andContractIdEqualTo(model.getContractId());
        }
        if (ObjectUtil.isNotNull(model.getPayType())) {
            criteria.andPayTypeEqualTo(model.getPayType());
        }
        return ex;
    }

    /**
     * 查询合同部分的信息
     *
     * @param contractId
     * @return
     */
    public PayLogDto getContractPart(String contractId) {

        ContractManage contractManage = contractManageMapper.selectByPrimaryKey(contractId);
        PayLogDto payLogDto = new PayLogDto();
        payLogDto.setContractNumber(contractManage.getContractNumber());
        payLogDto.setBuyerName(contractManage.getBuyerName());
        payLogDto.setHouseName(contractManage.getHouseName());
        payLogDto.setPaymentMethod(contractManage.getPaymentMethod());
//        payLogDto.setDeadLine(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD, contractManage.getDeadline()));
        // 查询已收房款
        String receivedMoneyStr = extendMapper.getMoneyByContractId(contractId, 1,null);
        BigDecimal receivedMoney = new BigDecimal(receivedMoneyStr);
        // 欠款
        BigDecimal arrears = BigDecimal.ZERO;
        // 应收
        BigDecimal receivableMoney = BigDecimal.ZERO;
        if (ObjectUtil.isNotNull(contractManage.getBuyerMoney())) {
            arrears = contractManage.getBuyerMoney().subtract(receivedMoney);
            receivableMoney = contractManage.getBuyerMoney();
        }
        payLogDto.setReceivableMoney(receivableMoney);
        payLogDto.setReceivedMoney(receivedMoney);
        payLogDto.setArrears(arrears);
        payLogDto.setBankName(contractManage.getBankName());
        payLogDto.setBankNumber(contractManage.getBankNumber());
        return payLogDto;
    }

    public PayLog getByContractId(String contractId, Integer payType) {

        PayLogExample payLogExample = new PayLogExample();
        payLogExample.createCriteria().andContractIdEqualTo(contractId).andPayTypeEqualTo(payType).andStatusEqualTo(0);
        List<PayLog> payLogs = modelMapper.selectByExample(payLogExample);
        return payLogs.get(0);

    }

    public PayLogDto getById(String id) {
        PayLog payLog = selectByPrimaryKey(id);
        String username = "";
        if (StringUtils.isNotEmpty(payLog.getCreatedBy())) {
            FrameUser frameUser = frameUserService.selectByPrimaryKey(payLog.getCreatedBy());
            username = frameUser == null ? "" : frameUser.getTruename();
        }
        PayLogDto result = new PayLogDto();
        result.setCreatedName(username);
        BeanUtil.copyProperties(payLog, result);
        return result;
    }

    /**
     * 根据合同id 查询收款
     *
     * @param contractId
     * @param payType
     * @return
     */
    public String getMoneyByContractId(String contractId, Integer payType, Integer paymentStatus) {
        String money = extendMapper.getMoneyByContractId(contractId, payType, paymentStatus);
        return money;
    }


    /**
     * 收款收据
     *
     * @param id
     * @param response
     */
    public void downLoadReceipt(String id, HttpServletResponse response) {

        PayLog payLog = selectByPrimaryKey(id);
        Integer contentType = payLog.getContentType();
        String sealPath = "/home/project/settleDown/tem/seal.png";
        String filePath = "";
        String fileName = "";
        switch (contentType) {
            case 1: // 定金
                filePath = "/home/project/settleDown/tem/receipt.docx";
                fileName = "定金收据.docx";
                break;
            case 2: // 首付
                filePath = "/home/project/settleDown/tem/receipt.docx";
                fileName = "首付收据.docx";
                break;
            case 3: // 银行按揭
                filePath = "/home/project/settleDown/tem/receipt.docx";
                fileName = "银行按揭收据.docx";
                break;
            case 8: // 首款-定金转
                filePath = "/home/project/settleDown/tem/receipt.docx";
                fileName = "首款-定金转.docx";
                break;
            case 9: // 房款-定金转
                filePath = "/home/project/settleDown/tem/receipt.docx";
                fileName = "房款-定金转.docx";
                break;
        }

        String customerManagementId = payLog.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        String name = "";
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            name = buyer.getName();
        }

        ParkRoomResponse roomResponse = roomExtendMapper.getFullName(payLog.getHouseId());
        String fullName = roomResponse.getGroupName()
                + "-"
                + roomResponse.getDiscName()
                + roomResponse.getBuildName()
                + roomResponse.getRoomNo();

        String collectionTime = "";
        try {
            collectionTime = DateUtils.parseDateToStr(DateUtils.DATE_CH, payLog.getPayTime());
        }catch (Exception e){
            log.error("日期转换异常:{}",e.getMessage());
        }
        String content = "";
        String paymentMethod = "";
        String contentTypeLabel = sysDictService.getDictLabel("CONTENT_TYPE", payLog.getContentType().toString());
        String paymentMethodLabel = sysDictService.getDictLabel("COLLECTION_METHODS", payLog.getPaymentMethod().toString());
        if (StringUtils.isNotEmpty(contentTypeLabel)) {
            content = contentTypeLabel;
        }
        if (StringUtils.isNotEmpty(paymentMethodLabel)) {
            paymentMethod = paymentMethodLabel;
        }
        String receivedAmount = payLog.getPayMoney().toString();
        String capital = MoneyUtils.amountConversion(receivedAmount);
        String serialNumber = payLog.getSerialNumber();

        Map<String, Object> data = new HashMap<>();
        Map<String, Object> picData = new HashMap<>();
        Map<String, Object> sealData = new HashMap<>();
        Map<String, Object> settingData = new HashMap<>();
        Map<String, Object> settingInfoData;

        data.put("${name}", name);
        data.put("${collectionTime}", collectionTime);
        data.put("${fullName}", fullName);
        data.put("${content}", content);
        data.put("${paymentMethod}", paymentMethod);
        data.put("${capital}", capital);
        data.put("${receivedAmount}", receivedAmount);
        data.put("${serialNumber}", serialNumber);
        sealData.put("${seal}", sealPath);
        settingInfoData = new HashMap<>();
        settingInfoData.put("leftOffset", "50");
        settingInfoData.put("topOffset", "0");
        settingData.put("${seal}",settingInfoData);

        ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData, settingData);


    }

    /**
     * 收款凭证推送
     * @param id
     * @return
     */
    public AjaxResult sendVoucher(String id) {

        // 银行流水id
//        String pkInformerId = model.getPkInformerId();
        PayLog payLog = selectByPrimaryKey(id);
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(payLog.getCustomerManagementId());
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagement.getId());
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        Buyer buyer = buyers.get(0);

        String pk_subjcode = "";
        // 定金
        if(payLog.getContentType() == 1){
            pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ001;
        }
        // 首付
        if(payLog.getContentType() == 2){
            pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ002;
        }
        // 银行按揭
        if(payLog.getContentType() == 3){
            pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ006;
        }
        // 专项维修资金
        if(payLog.getContentType() == 4){
            pk_subjcode = InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ015;
        }


        JSONObject json = new JSONObject();
        JSONObject ar_gatherbill = new JSONObject();
        String srcsystemid = id;
        String billdate = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date());
        ar_gatherbill.put("srcsystemid", srcsystemid); //外系统标识,必选
        ar_gatherbill.put("pk_tradetype", InvoiceConstant.INVOICE_BUSINESS_F2_Cxx_06); //交易类型,必选
        ar_gatherbill.put("pk_org", InvoiceUtil.pk_org); //组织编码,必选
        ar_gatherbill.put("billdate", billdate); //单据日期
        ar_gatherbill.put("def2", srcsystemid); //外系统单据号，必填，
        json.put("ar_gatherbill", ar_gatherbill);

        JSONArray ar_gatheritem = new JSONArray();
        JSONObject ar_gatheritemJson = new JSONObject();
        ar_gatheritemJson.put("srcsystemid", srcsystemid); //外系统子表标识
        ar_gatheritemJson.put("objtype", "0"); //0=客户，1=供应商--判断customer和supplier是否必填，必选
        ar_gatheritemJson.put("customer", buyer.getIdentityCard()); //客商的纳税人识别号
        ar_gatheritemJson.put("money_cr", payLog.getPayMoney()); //金额，必选
        ar_gatheritemJson.put("taxrate", "10"); //税率,必选，数字——3/10/6````
        ar_gatheritemJson.put("pk_balatype", "0"); //结算方式,必选，看云文档结算方式
        ar_gatheritemJson.put("pk_recpaytype", "001"); //付款业务类型传默认值“001”
        ar_gatheritemJson.put("prepay", "0"); //收款性质应收款时传"0",收款性质预收款时传"1"
        ar_gatheritemJson.put("recaccount", InvoiceUtil.bank_number); //收款银行账号,必填
//        ar_gatheritemJson.put("invoiceno","fphm"); //发票号码，非必选
        ar_gatheritemJson.put("scomment", buyer.getName()); //客户名称，必选
        ar_gatheritemJson.put("pk_subjcode", pk_subjcode); //收款类型
        ar_gatheritemJson.put("checktype", InvoiceConstant.INVOICE_RECEIPT_TYPE_3); //票据类型，必选，看云文档票据类型
        ar_gatheritemJson.put("checkno", payLog.getSerialNumber()); //票据号，必选
        ar_gatheritemJson.put("def9", InvoiceConstant.INVOICE_BUSINESS_FORMAT_001); //业态，必选，看云文档业态
        ar_gatheritemJson.put("def11", payLog.getHouseId()); //转出前房号编码，收除了诚意金之外的首期等需要选
        // TODO: 2024/11/14 具体编码应该是根据小区来的
        ar_gatheritemJson.put("def10", InvoiceConstant.INVOICE_PROJECT_CODE_mwjyAdk); //项目档案编码，看云文档项目档案对照
//        ar_gatheritemJson.put("top_billid", pkInformerId); //到账通知生单时,（银行流水中返回的id，见5.2.10到账通知认领查询pk_informer字段）
        ar_gatheritem.add(ar_gatheritemJson);
        json.put("ar_gatheritem", ar_gatheritem);

        try {
            String result = InvoiceUtil.gatheringbillAdd(json);
            JSONObject jsonObject = JSONObject.parseObject(result);
            if(jsonObject.getString("success").equals("true")){
                payLog.setNcJson(result);
                updateByPrimaryKeySelective(payLog);
            }

        } catch (Exception e) {
            log.error("各种凭证推送报错",e);
            return AjaxResult.error(e.getMessage());
        }
        return AjaxResult.success();
    }


    /**
     * 转首款 转房款
     * @param id
     * @param type
     * @return
     */
    @Transactional
    public int convertType(String id,Integer type){
        PayLog payLog = selectByPrimaryKey(id);
        // 1-意向金转定金，2-定金转首款，3-定金转房款
        Integer convertType = null;
        // 转首款
        if(type == 1){
            payLog.setContentType(8);
            payLog.setPaymentStatus(3);
            convertType = 2;
        }
        // 转房款
        if(type == 2){
            payLog.setContentType(9);
            payLog.setPaymentStatus(4);
            convertType = 3;
        }
        int result = modelMapper.updateByPrimaryKeySelective(payLog);

        // 新增转换记录
        ParkRoom parkRoom = parkRoomMapper.selectByPrimaryKey(payLog.getHouseId());
        ConvertRecord convertRecord = new ConvertRecord();
        BeanUtil.copyProperties(payLog, convertRecord);
        convertRecord.setGroupId(parkRoom.getGroupId());
        convertRecord.setDiscId(parkRoom.getDiscId());
        convertRecord.setId(IdUtil.simpleUUID());
        convertRecord.setBusinessId(payLog.getId());
        convertRecord.setBusinessType("pay_log");
        convertRecord.setConvertDate(new Date());
        convertRecord.setConvertType(convertType);
        convertRecord.setCreatedAt(new Date());
        convertRecord.setCreatedBy(ShiroUtils.getUserId());
        convertRecordMapper.insertSelective(convertRecord);

        return result;
    }


    public int delete(String id){

        PayLog payLog = modelMapper.selectByPrimaryKey(id);
        payLog.setStatus(1);
        int result = modelMapper.updateByPrimaryKeySelective(payLog);
        //删除票据管理里对应的几率
        ReceiptManageExample receiptManageExample = new ReceiptManageExample();
        receiptManageExample.createCriteria().andBusinessIdEqualTo(id);
        List<ReceiptManage> receiptManages = receiptManageService.selectByExample(receiptManageExample);
        if(CollectionUtils.isNotEmpty(receiptManages)){
            receiptManageService.deleteByPrimaryKey(receiptManages.get(0).getId());
        }
        return result;

    }




}
