package com.idea.pro.config;

import org.apache.catalina.connector.Connector;
import org.apache.coyote.http11.Http11NioProtocol;
import org.springframework.boot.web.embedded.tomcat.TomcatConnectorCustomizer;
import org.springframework.boot.web.embedded.tomcat.TomcatServletWebServerFactory;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Configuration
public class ProConfig {

    /**
     * Slow Attack 问题修正
     * @return
     */
    @Bean
    public TomcatServletWebServerFactory createServletFactory() {

        TomcatServletWebServerFactory tomcatFactory = new TomcatServletWebServerFactory();

        // tomcatFactory.setPort(8081);
        tomcatFactory.addConnectorCustomizers(new MyTomcatConnectorCustomizer());

        return tomcatFactory;
    }



    class MyTomcatConnectorCustomizer implements TomcatConnectorCustomizer {

        @Override
        public void customize(Connector connector) {

            Http11NioProtocol protocol = (Http11NioProtocol) connector.getProtocolHandler();

            //设置最大连接数
            // protocol.setMaxConnections(2000);

            //设置最大线程数
            // protocol.setMaxThreads(2000);

            protocol.setConnectionTimeout(8000); //就是这一句起作用了

        }
    }

    /**
     * Option问题过滤器修正
     * @return
     */
    @Bean
    public FilterRegistrationBean filterRegistration(){
        // 新建过滤器注册类
        FilterRegistrationBean registration = new FilterRegistrationBean();
        // 添加自定义 过滤器
        // 此处添加了Option问题过滤器
        registration.setFilter(new OptionFilter());
        // 设置过滤器的URL模式
        registration.addUrlPatterns("/*");
        //设置过滤器顺序
        registration.setOrder(1);
        return registration;
    }

    class OptionFilter extends OncePerRequestFilter {

        @Override
        protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
                throws ServletException, IOException {

            if (request.getMethod().equals("OPTIONS")) {

                response.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED);

            } else {

                filterChain.doFilter(request, response);

            }
        }
    }

//    @Bean
//    public ConfigurableServletWebServerFactory configurableServletWebServerFactory() {
//        TomcatServletWebServerFactory factory = new TomcatServletWebServerFactory();
//        factory.setProtocol("org.apache.coyote.http11.Http11NioProtocol");
//        factory.setPort(8850);
//        factory.addConnectorCustomizers(connector -> {
//            Http11NioProtocol protocol = (Http11NioProtocol) connector.getProtocolHandler();
//            protocol.setDisableUploadTimeout(false);
//            //protocol.setAcceptCount(200);
//            //protocol.setMaxConnections(200);
////            protocol.setMaxHeaderCount(20000);
////            protocol.setConnectionTimeout(20000);
//            protocol.setMaxHttpHeaderSize(65546);
////            protocol.setMaxSavePostSize(4194304);
//        });
//        return factory;
//    }
}